
# THIS FILE IS GENERATED FROM syned SETUP.PY
short_version = '1.0.13'
version = '1.0.13'
full_version = '1.0.13.dev0+f843317'
git_revision = 'f84331781f6e66d58681b3248a86a2a79b5bd384'
release = False

if not release:
    version = full_version
    short_version += ".dev"
